#include <netdb.h>
#include <sys/socket.h>
#include <poll.h>		/* XSI */
#include <stdio.h>
#include <stdlib.h>

#define DECIMAL 1
#define HEX 2

typedef struct
{
  const char *name;
  int         format;
  int         value;
} variable_t;

typedef struct vlist_s
{
  variable_t      var;
  struct vlist_s *next;
} vlist_t;

static vlist_t *push_variable(vlist_t **head, vlist_t *tail,
			      const char *name, int format, int value)
{
  vlist_t *c;
  if (*head == NULL) {
    *head = tail = (vlist_t *)malloc(sizeof(vlist_t));
    c = tail;
  } else {
    c = tail->next = (vlist_t *)malloc(sizeof(vlist_t));
  }
  c->var.name = name;
  c->var.format = format;
  c->var.value = value;
  c->next = NULL;
  return c;
}

static void dump_library(vlist_t *l)
{
  vlist_t *i;
  int indent = 0;
  fputs(";; -*- mode:scheme; coding:utf-8 -*-\n", stdout);
  fputs(";; This file is automatically generated. DON'T EDIT!\n", stdout);
  fputs("(library (usocket consts)\n", stdout);
  fputs("    (export", stdout);
  for (i = l; i; i = i->next) {
    int j;
    for (j = 0; j < indent; j++) {
      if (j == 0) fputs("\n", stdout);
      fputs(" ", stdout);
    }
    fprintf(stdout, " usocket:%s", i->var.name);
    indent = sizeof("    (export") - 1;
  }
  fputs(")\n", stdout);
  fputs("    (import (rnrs))\n", stdout);
  
  for (i = l; i; i = i->next) {
    if (i->var.format == DECIMAL) {
      fprintf(stdout, "(define usocket:%s %d)\n", i->var.name, i->var.value);
    } else if (i->var.format == HEX) {
      fprintf(stdout, "(define usocket:%s #x%x)\n", i->var.name, i->var.value);
    }
  }
  fputs(")\n", stdout);
}

int main(int argc, char **argv)
{
  vlist_t *head = NULL, *tail = NULL;

#define push_var(cvar)				\
  tail = push_variable(&head, tail, #cvar, DECIMAL, cvar)
#define push_hex_var(cvar)		\
  tail = push_variable(&head, tail, #cvar, HEX, cvar)

  /* AF_* */
  push_var(AF_UNSPEC);
  push_var(AF_INET);
  push_var(AF_INET6);
  /* SOCK_* */
  push_var(SOCK_STREAM);
  push_var(SOCK_DGRAM);
  /* AI_* */
  push_hex_var(AI_PASSIVE);
  push_hex_var(AI_CANONNAME);
  push_hex_var(AI_NUMERICHOST);
  push_hex_var(AI_NUMERICSERV);
  push_hex_var(AI_ALL);
  push_hex_var(AI_ADDRCONFIG);
  push_hex_var(AI_V4MAPPED);
  /* IP_* */
  push_var(IPPROTO_IP);
  push_var(IPPROTO_TCP);
  push_var(IPPROTO_UDP);
  /* MSG_* */
  push_hex_var(MSG_OOB);
  push_hex_var(MSG_PEEK);
  push_hex_var(MSG_WAITALL);
#ifdef MSG_NOSIGNAL
  push_hex_var(MSG_NOSIGNAL);
#else
  tail = push_variable(&head, tail, "MSG_NOSIGNAL", DECIMAL, 0);
#endif
  /* SHUT_* */
  push_var(SHUT_RD);
  push_var(SHUT_WR);
  push_var(SHUT_RDWR);

  /* SOMAXCONN */
  push_var(SOMAXCONN);

  /* POLL */
  push_hex_var(POLLIN);	/* POLLRDNORM | POLLRDBAND but for convenience */
  push_hex_var(POLLRDNORM);
  push_hex_var(POLLRDBAND);
  push_hex_var(POLLPRI);
  push_hex_var(POLLOUT);
  push_hex_var(POLLWRNORM);
  push_hex_var(POLLWRBAND);
  push_hex_var(POLLERR);
  push_hex_var(POLLHUP);
  push_hex_var(POLLNVAL);
  /* some of we can't handle it from Scheme without this things */
  tail = push_variable(&head, tail, "size-of-sockaddr-storage", DECIMAL,
		       sizeof(struct sockaddr_storage));
  tail = push_variable(&head, tail, "size-of-socklen_t", DECIMAL,
		       sizeof(socklen_t));
  
  dump_library(head);
  /* we don't care freeing the allocated memory since this tool is too small
     to care */
  return 0;
}
